from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.identity_driver import IdentityDriverAttributes

class ActiveDirectoryIdentityDriverAttributes(IdentityDriverAttributes, metaclass=IterableMeta):
    __config_class__ = "ActiveDirectory Identity Driver"
    base_dn = Attribute('Base DN', min_version='21.4')
    computer_class_name = Attribute('Computer Class Name', min_version='21.4')
    computer_query_expression = Attribute('Computer Query Expression', min_version='21.4')
    configuration = Attribute('Configuration', min_version='21.4')
    container_class_name = Attribute('Container Class Name', min_version='21.4')
    container_query_expression = Attribute('Container Query Expression', min_version='21.4')
    credential = Attribute('Credential', min_version='21.4')
    dsn = Attribute('DSN', min_version='21.4')
    distribution_list_class_name = Attribute('Distribution List Class Name', min_version='21.4')
    distribution_list_query_expression = Attribute('Distribution List Query Expression', min_version='21.4')
    friendly_name = Attribute('Friendly Name', min_version='21.4')
    group_class_name = Attribute('Group Class Name', min_version='21.4')
    group_query_expression = Attribute('Group Query Expression', min_version='21.4')
    host = Attribute('Host', min_version='21.4')
    no_rediscovery_on = Attribute('No Rediscovery On', min_version='21.4')
    port = Attribute('Port', min_version='21.4')
    resolve_nested_groups = Attribute('Resolve Nested Groups', min_version='21.4')
    search_filter = Attribute('Search Filter', min_version='21.4')
    search_root = Attribute('Search Root', min_version='21.4')
    secure = Attribute('Secure', min_version='21.4')
    timeout = Attribute('Timeout', min_version='21.4')
    user_class_name = Attribute('User Class Name', min_version='21.4')
    user_query_expression = Attribute('User Query Expression', min_version='21.4')
    vault_id = Attribute('Vault Id', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class MetadataCategoryAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Metadata Category"
    label_text = Attribute('Label Text')
    localization = Attribute('Localization')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class KeynectisSequoiaCAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Keynectis Sequoia CA"
    csr_format_name = Attribute('CSR Format Name', min_version='21.4')
    csr_name = Attribute('CSR Name', min_version='21.4')
    customer = Attribute('Customer', min_version='21.4')
    encryption_certificate = Attribute('Encryption Certificate', min_version='21.4')
    enrollment_mode = Attribute('Enrollment Mode', min_version='21.4')
    enterprise = Attribute('Enterprise', min_version='21.4')
    fields = Attribute('Fields', min_version='21.4')
    offer = Attribute('Offer', min_version='21.4')
    uri = Attribute('URI', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import stats

class _Stats(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Stats')
        self.GetCounters = self._GetCounters(api_obj=self._api_obj, url=f'{self._url}/GetCounters')
        self.Query = self._Query(api_obj=self._api_obj, url=f'{self._url}/Query')

        self.CreateCounter = self._CreateCounter(api_obj=api_obj, url=f'{self._url}/CreateCounter')
        self.DeleteCounter = self._DeleteCounter(api_obj=api_obj, url=f'{self._url}/DeleteCounter')
        self.RenameCounter = self._RenameCounter(api_obj=api_obj, url=f'{self._url}/RenameCounter')
        self.UpdateCounter = self._UpdateCounter(api_obj=api_obj, url=f'{self._url}/UpdateCounter')
        self.GetCounter = self._GetCounter(api_obj=api_obj, url=f'{self._url}/GetCounter')
        self.CounterContainers = self._CounterContainers(api_obj=api_obj, url=f'{self._url}/CounterContainers')
        self.QueryTags = self._QueryTags(api_obj=api_obj, url=f'{self._url}/QueryTags')

    class _CreateCounter(WebSdkEndpoint):
        def post(
            self,
            stats_type: int = None,
            name: str = None,
            description: str = None,
            metadata_aname: str = None,
            metadata_bname: str = None,
            metadata_cname: str = None,
            value_description: str = None,
            sensitive: bool = None,
        ):
            body = {
                "StatsType"       : stats_type,
                "Name"            : name,
                "Description"     : description,
                "MetadataAName"   : metadata_aname,
                "MetadataBName"   : metadata_bname,
                "MetadataCName"   : metadata_cname,
                "ValueDescription": value_description,
                "Sensitive"       : sensitive,
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _DeleteCounter(WebSdkEndpoint):
        def post(self, stats_type: int):
            body = {
                "StatsType": stats_type,
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RenameCounter(WebSdkEndpoint):
        def post(self, stats_type: int, name: str):
            body = {
                "StatsType": stats_type,
                "Name"     : name,
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _UpdateCounter(WebSdkEndpoint):
        def post(
            self,
            stats_type: int = None,
            description: str = None,
            metadata_aname: str = None,
            metadata_bname: str = None,
            metadata_cname: str = None,
            value_description: str = None,
            sensitive: bool = None,
        ):
            body = {
                "StatsType"       : stats_type,
                "Description"     : description,
                "MetadataAName"   : metadata_aname,
                "MetadataBName"   : metadata_bname,
                "MetadataCName"   : metadata_cname,
                "ValueDescription": value_description,
                "Sensitive"       : sensitive,
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetCounter(WebSdkEndpoint):
        def post(self, stats_type: int):
            body = {
                "StatsType": stats_type,
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')
                counter: stats.Counter = ApiField(alias='Counter')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _CounterContainers(WebSdkEndpoint):
        def post(self):
            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')
                counter_containers: list[stats.CounterContainer] = ApiField(alias='CounterContainers')

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _QueryTags(WebSdkEndpoint):
        def post(self, stats_type: int = None, filter: str = None, tag_number: int = None):
            body = {
                "StatsType": stats_type,
                "Filter"   : filter,
                "TagNumber": tag_number,
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')
                tag_a_values: list[str] = ApiField(alias='TagAValues')
                tag_b_values: list[str] = ApiField(alias='TagBValues')
                tag_c_values: list[str] = ApiField(alias='TagCValues')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetCounters(WebSdkEndpoint):
        def post(self):
            class Output(WebSdkOutputModel):
                counters: list[stats.Counter] = ApiField(alias='Counters', default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _Query(WebSdkEndpoint):
        def post(
            self, stats_type: str, tier: int, max_points: int, group_by_a: bool = None, group_by_b: bool = None,
            group_by_c: bool = None, filter_a: str = None, filter_b: str = None, filter_c: str = None
        ):
            body = {
                'StatsType': stats_type,
                'Tier'     : tier,
                'MaxPoints': max_points,
                'GroupByA' : group_by_a,
                'GroupByB' : group_by_b,
                'GroupByC' : group_by_c,
                'FilterA'  : filter_a,
                'FilterB'  : filter_b,
                'FilterC'  : filter_c
            }

            class Output(WebSdkOutputModel):
                results: list[stats.Result] = ApiField(alias='Results', default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import secret_store

class _SecretStore(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/SecretStore')
        self.Add = self._Add(api_obj=self._api_obj, url=f'{self._url}/Add')
        self.Associate = self._Associate(api_obj=self._api_obj, url=f'{self._url}/Associate')
        self.Dissociate = self._Dissociate(api_obj=self._api_obj, url=f'{self._url}/Dissociate')
        self.EncryptionKeysInUse = self._EncryptionKeysInUse(
            api_obj=self._api_obj,
            url=f'{self._url}/EncryptionKeysInUse'
        )
        self.Lookup = self._Lookup(api_obj=self._api_obj, url=f'{self._url}/Lookup')
        self.LookupAllAssociationsbyVaultid = self._LookupAllAssociationsbyVaultid(
            api_obj=self._api_obj,
            url=f'{self._url}/LookupAllAssociationsbyVaultid'
        )
        self.LookupByAssociation = self._LookupByAssociation(
            api_obj=self._api_obj,
            url=f'{self._url}/LookupByAssociation'
        )
        self.LookupAssociationbyVaultID = self._LookupAssociationbyVaultID(
            api_obj=self._api_obj,
            url=f'{self._url}/LookupAssociationbyVaultID'
        )
        self.LookupByOwner = self._LookupByOwner(api_obj=self._api_obj, url=f'{self._url}/LookupByOwner')
        self.LookupByVaultType = self._LookupByVaultType(api_obj=self._api_obj, url=f'{self._url}/LookupByVaultType')
        self.Mutate = self._Mutate(api_obj=self._api_obj, url=f'{self._url}/Mutate')
        self.OrphanLookup = self._OrphanLookup(api_obj=self._api_obj, url=f'{self._url}/OrphanLookup')
        self.OwnerAdd = self._OwnerAdd(api_obj=self._api_obj, url=f'{self._url}/OwnerAdd')
        self.OwnerDelete = self._OwnerDelete(api_obj=self._api_obj, url=f'{self._url}/OwnerDelete')
        self.OwnerLookup = self._OwnerLookup(api_obj=self._api_obj, url=f'{self._url}/OwnerLookup')
        self.Retrieve = self._Retrieve(api_obj=self._api_obj, url=f'{self._url}/Retrieve')

        self.AvailableNamespaces = self._AvailableNamespaces(
            api_obj=self._api_obj,
            url=f'{self._url}/AvailableNamespaces'
        )
        self.RotateKey = self._RotateKey(api_obj=self._api_obj, url=f'{self._url}/RotateKey')

    class _Add(WebSdkEndpoint):
        def post(self, base_64_data: str, keyname: str, namespace: str, owner: str, vault_type: int):
            body = {
                'Base64Data': base_64_data,
                'Keyname'   : keyname,
                'Namespace' : namespace,
                'Owner'     : owner,
                'VaultType' : vault_type
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_id: int = ApiField(alias='VaultID')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Associate(WebSdkEndpoint):
        def post(
            self,
            name: str,
            vault_id: int,
            date_value: str = None,
            int_value: int = None,
            string_value: str = None
        ):
            body = {
                'Name'       : name,
                'VaultID'    : vault_id,
                'DateValue'  : date_value,
                'IntValue'   : int_value,
                'StringValue': string_value
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _AvailableNamespaces(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                namespaces: list[str] = ApiField(alias='Namespaces')
                encryption_keys: list[str] = ApiField(alias='EncryptionKeys')
                vault_id: int = ApiField(alias='VaultId')
                vault_ids: list[int] = ApiField(alias='VaultIds')
                vault_type: int = ApiField(alias='VaultType')
                owners: list[str] = ApiField(alias='Owners')
                result: int = ApiField(alias='Result')
                base_64_data: str = ApiField(alias='Base64Data')
                name: str = ApiField(alias='Name')
                value: str = ApiField(alias='Value')
                typed_name_values: list[secret_store.TypedNameValues] = ApiField(alias='TypedNameValues')

            return generate_output(output_cls=Output, response=self._get())

    class _Dissociate(WebSdkEndpoint):
        def post(
            self,
            vault_id: int,
            int_value: int = None,
            name: str = None,
            string_value: str = None,
            date_value: int = None
        ):
            body = {
                'VaultID'    : vault_id,
                'IntValue'   : int_value,
                'Name'       : name,
                'StringValue': string_value,
                'DateValue'  : date_value
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _EncryptionKeysInUse(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                encryption_keys: list[str] = ApiField(default_factory=list, alias='EncryptionKeys')
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._get())

    class _Lookup(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_ids: list[int] = ApiField(default_factory=list, alias='VaultIDs')

            return generate_output(output_cls=Output, response=self._get())

    class _LookupAllAssociationsbyVaultid(WebSdkEndpoint):
        def post(self, vault_id: int):
            body = {
                'VaultID': vault_id
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                typed_name_values: list[secret_store.TypedNameValues] = ApiField(
                    default_factory=list,
                    alias='TypedNameValues'
                )

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _LookupByAssociation(WebSdkEndpoint):
        def post(self, name: str, int_value: int = None, string_value: str = None, date_value: int = None):
            body = {
                'Name'       : name,
                'IntValue'   : int_value,
                'StringValue': string_value,
                'DateValue'  : date_value
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_ids: list[int] = ApiField(default_factory=list, alias='VaultIDs')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _LookupAssociationbyVaultID(WebSdkEndpoint):
        def post(self, vault_id: int, name: str = None):
            body = {
                'VaultID': vault_id,
                'Name'   : name
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                value: str = ApiField(alias='Value')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _LookupByOwner(WebSdkEndpoint):
        def post(self, namespace: str, owner: str, vault_type: str = None):
            body = {
                'Namespace': namespace,
                'Owner'    : owner,
                'VaultType': vault_type
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_ids: list[int] = ApiField(default_factory=list, alias='VaultIDs')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _LookupByVaultType(WebSdkEndpoint):
        def post(self, vault_type: int):
            body = {
                'VaultType': vault_type
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_ids: list[int] = ApiField(default_factory=list, alias='VaultIDs')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Mutate(WebSdkEndpoint):
        def post(self, vault_id: int, vault_type: int):
            body = {
                'VaultID'  : vault_id,
                'VaultType': vault_type
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _OrphanLookup(WebSdkEndpoint):
        def post(self, vault_type: int):
            body = {
                'VaultType': vault_type
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_ids: list[int] = ApiField(default_factory=list, alias='VaultIDs')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _OwnerAdd(WebSdkEndpoint):
        def post(self, namespace: str, owner: str, vault_id: int):
            body = {
                'Namespace': namespace,
                'Owner'    : owner,
                'VaultId'  : vault_id
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _OwnerDelete(WebSdkEndpoint):
        def post(self, namespace: str, owner: str, vault_id: int = None):
            body = {
                'Namespace': namespace,
                'Owner'    : owner,
                'VaultId'  : vault_id
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _OwnerLookup(WebSdkEndpoint):
        def post(self, namespace: str, vault_id: int):
            body = {
                'Namespace': namespace,
                'VaultID'  : vault_id
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                owners: list[str] = ApiField(default_factory=list, alias='Owners')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Retrieve(WebSdkEndpoint):
        def post(self, vault_id: int):
            body = {
                'VaultID': vault_id
            }

            class Output(WebSdkOutputModel):
                base_64_data: str = ApiField(alias='Base64Data')
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_type: str = ApiField(alias='VaultType')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RotateKey(WebSdkEndpoint):
        def Guid(self, guid: str):
            return self._Guid(api_obj=self._api_obj, url=f'{self._url}/{guid}')

        class _Guid(WebSdkEndpoint):
            def get(self, new_key_name: str):
                params = {
                    "newkeyname": new_key_name,
                }

                class Output(WebSdkOutputModel):
                    namespaces: list[str] = ApiField(alias='Namespaces')
                    encryption_keys: list[str] = ApiField(alias='EncryptionKeys')
                    vault_id: int = ApiField(alias='VaultId')
                    vault_ids: list[int] = ApiField(alias='VaultIds')
                    vault_type: int = ApiField(alias='VaultType')
                    owners: list[str] = ApiField(alias='Owners')
                    result: int = ApiField(alias='Result')
                    base_64_data: str = ApiField(alias='Base64Data')
                    name: str = ApiField(alias='Name')
                    value: str = ApiField(alias='Value')
                    typed_name_values: list[secret_store.TypedNameValues] = ApiField(alias='TypedNameValues')

                return generate_output(output_cls=Output, response=self._get(params=params))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class RiverbedSteelHeadAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Riverbed SteelHead"
    certificate_type = Attribute('Certificate Type', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    install_chain = Attribute('Install Chain', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    replace_existing = Attribute('Replace Existing', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class ConnectDirectAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "ConnectDirect"
    certificate_label = Attribute('Certificate Label', min_version='21.4')
    certificate_only = Attribute('Certificate Only', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    node_name = Attribute('Node Name', min_version='21.4')
    protocol = Attribute('Protocol', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.report_base import ReportBaseAttributes
from pyvenafi.tpp.attributes.report_filter_base import ReportFilterBaseAttributes

class ReportClientCertificateTrendsReportAttributes(
    ReportBaseAttributes,
    ReportFilterBaseAttributes,
    metaclass=IterableMeta
):
    __config_class__ = "Report:Client Certificate Trends Report"



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    CloudApiEndpoint,
    CloudApiOutputModel,
    generate_output,
)
from pyvenafi.cloud.api.models import caoperations_service
from typing import Literal
from uuid import UUID

class _caoperations_service:
    def __init__(self, api_obj):
        self.v1 = self._v1(api_obj=api_obj)

    class _v1(CloudApiEndpoint):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='v1')
            self.builtinca = self._builtinca(api_obj=self._api_obj, url=f'{self._url}/builtinca')
            self.certificateauthorities = self._certificateauthorities(
                api_obj=self._api_obj,
                url=f'{self._url}/certificateauthorities'
            )
            self.certificateissuingtemplates = self._certificateissuingtemplates(
                api_obj=self._api_obj, url=f'{self._url}/certificateissuingtemplates'
            )
            self.certificaterequests = self._certificaterequests(
                api_obj=self._api_obj,
                url=f'{self._url}/certificaterequests'
            )
            self.certificates = self._certificates(api_obj=self._api_obj, url=f'{self._url}/certificates')

        class _builtinca(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.ca = self._ca(api_obj=self._api_obj, url=f'{self._url}/ca')
                self.cachain = self._cachain(api_obj=self._api_obj, url=f'{self._url}/cachain')
                self.crl = self._crl(api_obj=self._api_obj, url=f'{self._url}/crl')

            class _ca(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def CANAME(self, caname: str):
                    return self._CANAME(api_obj=self._api_obj, url=f'{self._url}/{caname}')

                class _CANAME(CloudApiEndpoint):
                    def get(self):
                        class Output(CloudApiOutputModel):
                            str: str

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'str'
                            }
                        )

            class _cachain(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def CANAME(self, caname: str):
                    return self._CANAME(api_obj=self._api_obj, url=f'{self._url}/{caname}')

                class _CANAME(CloudApiEndpoint):
                    def get(self):
                        class Output(CloudApiOutputModel):
                            str: str

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'str'
                            }
                        )

            class _crl(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def CANAME(self, caname: str):
                    return self._CANAME(api_obj=self._api_obj, url=f'{self._url}/{caname}')

                class _CANAME(CloudApiEndpoint):
                    def get(self):
                        class Output(CloudApiOutputModel):
                            str: str

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'str'
                            }
                        )

        class _certificateauthorities(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)

            def CERTIFICATEAUTHORITY(self, certificateauthority: str):
                return self._CERTIFICATEAUTHORITY(api_obj=self._api_obj, url=f'{self._url}/{certificateauthority}')

            def get(
                self,
                includeSystemGenerated: bool,
                issuanceCertificateType: Literal['ALL', 'END_ENTITY', 'ISSUER']
            ):
                data = {
                    'includeSystemGenerated' : includeSystemGenerated,
                    'issuanceCertificateType': issuanceCertificateType,
                }

                class Output(CloudApiOutputModel):
                    CertificateAuthorityResponse: caoperations_service.CertificateAuthorityResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params=data),
                    rc_mapping={
                        200: 'CertificateAuthorityResponse'
                    }
                )

            class _CERTIFICATEAUTHORITY(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.accounts = self._accounts(api_obj=self._api_obj, url=f'{self._url}/accounts')

                class _accounts(CloudApiEndpoint):
                    def __init__(self, *args, **kwargs):
                        super().__init__(*args, **kwargs)
                        self.configuration = self._configuration(
                            api_obj=self._api_obj,
                            url=f'{self._url}/configuration'
                        )
                        self.connection = self._connection(api_obj=self._api_obj, url=f'{self._url}/connection')
                        self.credentials = self._credentials(api_obj=self._api_obj, url=f'{self._url}/credentials')

                    def ACCOUNTID(self, accountid: str):
                        return self._ACCOUNTID(api_obj=self._api_obj, url=f'{self._url}/{accountid}')

                    def ID(self, id: str):
                        return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                    def get(
                        self,
                        edgeInstanceId: UUID,
                        includeObsoleteOptionsDetails: bool,
                        includeOptionsDetails: bool,
                        reloadOptionsDetails: bool
                    ):
                        data = {
                            'edgeInstanceId'               : edgeInstanceId,
                            'includeObsoleteOptionsDetails': includeObsoleteOptionsDetails,
                            'includeOptionsDetails'        : includeOptionsDetails,
                            'reloadOptionsDetails'         : reloadOptionsDetails,
                        }

                        class Output(CloudApiOutputModel):
                            CertificateAuthorityAccountResponse: caoperations_service.CertificateAuthorityAccountResponse

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params=data),
                            rc_mapping={
                                200: 'CertificateAuthorityAccountResponse'
                            }
                        )

                    def post(
                        self,
                        CertificateAuthorityAccountRequest: caoperations_service.CertificateAuthorityAccountRequest
                    ):
                        data = {**CertificateAuthorityAccountRequest.dict()}

                        class Output(CloudApiOutputModel):
                            CertificateAuthorityAccountResponse: caoperations_service.CertificateAuthorityAccountResponse

                        return generate_output(
                            output_cls=Output,
                            response=self._post(data=data),
                            rc_mapping={
                                201: 'CertificateAuthorityAccountResponse',
                                202: 'CertificateAuthorityAccountResponse'
                            }
                        )

                    class _ACCOUNTID(CloudApiEndpoint):
                        def __init__(self, *args, **kwargs):
                            super().__init__(*args, **kwargs)
                            self.domains = self._domains(api_obj=self._api_obj, url=f'{self._url}/domains')
                            self.importoptions = self._importoptions(
                                api_obj=self._api_obj,
                                url=f'{self._url}/importoptions'
                            )
                            self.operations = self._operations(api_obj=self._api_obj, url=f'{self._url}/operations')
                            self.productoptions = self._productoptions(
                                api_obj=self._api_obj,
                                url=f'{self._url}/productoptions'
                            )

                        class _domains(CloudApiEndpoint):
                            def __init__(self, *args, **kwargs):
                                super().__init__(*args, **kwargs)

                            def ID(self, id: str):
                                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                            def get(self):
                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountDomainInformation: caoperations_service.CertificateAuthorityAccountDomainInformation

                                return generate_output(
                                    output_cls=Output,
                                    response=self._get(params={}),
                                    rc_mapping={
                                        200: 'CertificateAuthorityAccountDomainInformation'
                                    }
                                )

                            def post(
                                self,
                                CertificateAuthorityAccountDomainRequest: caoperations_service.CertificateAuthorityAccountDomainRequest
                            ):
                                data = {**CertificateAuthorityAccountDomainRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountDomainInformation: caoperations_service.CertificateAuthorityAccountDomainInformation

                                return generate_output(
                                    output_cls=Output,
                                    response=self._post(data=data),
                                    rc_mapping={
                                        200: 'CertificateAuthorityAccountDomainInformation'
                                    }
                                )

                            class _ID(CloudApiEndpoint):
                                def get(self):
                                    class Output(CloudApiOutputModel):
                                        CertificateAuthorityAccountDomainInformation: caoperations_service.CertificateAuthorityAccountDomainInformation

                                    return generate_output(
                                        output_cls=Output,
                                        response=self._get(params={}),
                                        rc_mapping={
                                            200: 'CertificateAuthorityAccountDomainInformation'
                                        }
                                    )

                        class _importoptions(CloudApiEndpoint):
                            def __init__(self, *args, **kwargs):
                                super().__init__(*args, **kwargs)

                            def ID(self, id: str):
                                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                            def get(self):
                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountImportOptionResponse: caoperations_service.CertificateAuthorityAccountImportOptionResponse

                                return generate_output(
                                    output_cls=Output,
                                    response=self._get(params={}),
                                    rc_mapping={
                                        200: 'CertificateAuthorityAccountImportOptionResponse'
                                    }
                                )

                            def post(
                                self,
                                CertificateAuthorityAccountImportOptionRequest: caoperations_service.CertificateAuthorityAccountImportOptionRequest
                            ):
                                data = {**CertificateAuthorityAccountImportOptionRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityImportOptionInformation: caoperations_service.CertificateAuthorityImportOptionInformation

                                return generate_output(
                                    output_cls=Output,
                                    response=self._post(data=data),
                                    rc_mapping={
                                        201: 'CertificateAuthorityImportOptionInformation'
                                    }
                                )

                            class _ID(CloudApiEndpoint):
                                def delete(self):
                                    class Output(CloudApiOutputModel):
                                        pass

                                    return generate_output(output_cls=Output, response=self._delete(params={}))

                                def get(self):
                                    class Output(CloudApiOutputModel):
                                        CertificateAuthorityImportOptionInformation: caoperations_service.CertificateAuthorityImportOptionInformation

                                    return generate_output(
                                        output_cls=Output,
                                        response=self._get(params={}),
                                        rc_mapping={
                                            200: 'CertificateAuthorityImportOptionInformation'
                                        }
                                    )

                        class _operations(CloudApiEndpoint):
                            def post(self, CaOperationRequest: caoperations_service.CaOperationRequest):
                                data = {**CaOperationRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountInformation: caoperations_service.CertificateAuthorityAccountInformation

                                return generate_output(
                                    output_cls=Output,
                                    response=self._post(data=data),
                                    rc_mapping={
                                        200: 'CertificateAuthorityAccountInformation'
                                    }
                                )

                        class _productoptions(CloudApiEndpoint):
                            def __init__(self, *args, **kwargs):
                                super().__init__(*args, **kwargs)
                                self.testissuance = self._testissuance(
                                    api_obj=self._api_obj,
                                    url=f'{self._url}/testissuance'
                                )

                            def ID(self, id: str):
                                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                            def get(self):
                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityProductOptionResponse: caoperations_service.CertificateAuthorityProductOptionResponse

                                return generate_output(
                                    output_cls=Output,
                                    response=self._get(params={}),
                                    rc_mapping={
                                        200: 'CertificateAuthorityProductOptionResponse'
                                    }
                                )

                            def post(
                                self,
                                CertificateAuthorityProductOptionRequest: caoperations_service.CertificateAuthorityProductOptionRequest
                            ):
                                data = {**CertificateAuthorityProductOptionRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityProductOptionInformation: caoperations_service.CertificateAuthorityProductOptionInformation
                                    CertificateAuthorityProductOptionResponse: caoperations_service.CertificateAuthorityProductOptionResponse

                                return generate_output(
                                    output_cls=Output,
                                    response=self._post(data=data),
                                    rc_mapping={
                                        201: 'CertificateAuthorityProductOptionInformation',
                                        202: 'CertificateAuthorityProductOptionResponse'
                                    }
                                )

                            class _ID(CloudApiEndpoint):
                                def delete(self):
                                    class Output(CloudApiOutputModel):
                                        pass

                                    return generate_output(output_cls=Output, response=self._delete(params={}))

                                def get(self):
                                    class Output(CloudApiOutputModel):
                                        CertificateAuthorityProductOptionInformation: caoperations_service.CertificateAuthorityProductOptionInformation

                                    return generate_output(
                                        output_cls=Output,
                                        response=self._get(params={}),
                                        rc_mapping={
                                            200: 'CertificateAuthorityProductOptionInformation'
                                        }
                                    )

                            class _testissuance(CloudApiEndpoint):
                                def get(self):
                                    class Output(CloudApiOutputModel):
                                        CertificateAuthorityProductOptionsTestIssuanceResultsResponse: caoperations_service.CertificateAuthorityProductOptionsTestIssuanceResultsResponse

                                    return generate_output(
                                        output_cls=Output,
                                        response=self._get(params={}),
                                        rc_mapping={
                                            200: 'CertificateAuthorityProductOptionsTestIssuanceResultsResponse'
                                        }
                                    )

                                def post(
                                    self,
                                    CertificateAuthorityProductOptionsTestIssuanceRequest: caoperations_service.CertificateAuthorityProductOptionsTestIssuanceRequest
                                ):
                                    data = {**CertificateAuthorityProductOptionsTestIssuanceRequest.dict()}

                                    class Output(CloudApiOutputModel):
                                        pass

                                    return generate_output(output_cls=Output, response=self._post(data=data))

                    class _ID(CloudApiEndpoint):
                        def __init__(self, *args, **kwargs):
                            super().__init__(*args, **kwargs)
                            self.certificateimportstatus = self._certificateimportstatus(
                                api_obj=self._api_obj, url=f'{self._url}/certificateimportstatus'
                            )
                            self.issuingrules = self._issuingrules(
                                api_obj=self._api_obj,
                                url=f'{self._url}/issuingrules'
                            )
                            self.synchronisation = self._synchronisation(
                                api_obj=self._api_obj,
                                url=f'{self._url}/synchronisation'
                            )

                        def delete(self):
                            class Output(CloudApiOutputModel):
                                CertificateAuthorityAccountDeleteResponse: caoperations_service.CertificateAuthorityAccountDeleteResponse

                            return generate_output(
                                output_cls=Output,
                                response=self._delete(params={}),
                                rc_mapping={
                                    200: 'CertificateAuthorityAccountDeleteResponse'
                                }
                            )

                        def get(
                            self,
                            includeObsoleteOptionsDetails: bool,
                            includeOptionsDetails: bool,
                            reloadAccountDetails: bool,
                            reloadOptionsDetails: bool
                        ):
                            data = {
                                'includeObsoleteOptionsDetails': includeObsoleteOptionsDetails,
                                'includeOptionsDetails'        : includeOptionsDetails,
                                'reloadAccountDetails'         : reloadAccountDetails,
                                'reloadOptionsDetails'         : reloadOptionsDetails,
                            }

                            class Output(CloudApiOutputModel):
                                ExtendedCertificateAuthorityAccountInformation: caoperations_service.ExtendedCertificateAuthorityAccountInformation

                            return generate_output(
                                output_cls=Output,
                                response=self._get(params=data),
                                rc_mapping={
                                    200: 'ExtendedCertificateAuthorityAccountInformation'
                                }
                            )

                        def put(
                            self,
                            CertificateAuthorityAccountUpdateRequest: caoperations_service.CertificateAuthorityAccountUpdateRequest
                        ):
                            data = {**CertificateAuthorityAccountUpdateRequest.dict()}

                            class Output(CloudApiOutputModel):
                                CertificateAuthorityAccountResponse: caoperations_service.CertificateAuthorityAccountResponse

                            return generate_output(
                                output_cls=Output,
                                response=self._put(data=data),
                                rc_mapping={
                                    200: 'CertificateAuthorityAccountResponse'
                                }
                            )

                        class _certificateimportstatus(CloudApiEndpoint):
                            def put(self, CertificateImportRequest: caoperations_service.CertificateImportRequest):
                                data = {**CertificateImportRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountResponse: caoperations_service.CertificateAuthorityAccountResponse

                                return generate_output(
                                    output_cls=Output,
                                    response=self._put(data=data),
                                    rc_mapping={
                                        204: 'CertificateAuthorityAccountResponse'
                                    }
                                )

                        class _issuingrules(CloudApiEndpoint):
                            def get(self):
                                class Output(CloudApiOutputModel):
                                    CertificateIssuingTemplateRulesInformation: caoperations_service.CertificateIssuingTemplateRulesInformation

                                return generate_output(
                                    output_cls=Output,
                                    response=self._get(params={}),
                                    rc_mapping={
                                        200: 'CertificateIssuingTemplateRulesInformation'
                                    }
                                )

                        class _synchronisation(CloudApiEndpoint):
                            def put(self):
                                class Output(CloudApiOutputModel):
                                    ExtendedCertificateAuthorityAccountInformation: caoperations_service.ExtendedCertificateAuthorityAccountInformation

                                return generate_output(
                                    output_cls=Output,
                                    response=self._put(data={}),
                                    rc_mapping={
                                        200: 'ExtendedCertificateAuthorityAccountInformation'
                                    }
                                )

                    class _configuration(CloudApiEndpoint):
                        def __init__(self, *args, **kwargs):
                            super().__init__(*args, **kwargs)
                            self.validation = self._validation(api_obj=self._api_obj, url=f'{self._url}/validation')

                        class _validation(CloudApiEndpoint):
                            def post(
                                self,
                                CertificateAuthorityAccountConfigurationRequest: caoperations_service.CertificateAuthorityAccountConfigurationRequest
                            ):
                                data = {**CertificateAuthorityAccountConfigurationRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountResponse: caoperations_service.CertificateAuthorityAccountResponse

                                return generate_output(
                                    output_cls=Output,
                                    response=self._post(data=data),
                                    rc_mapping={
                                        200: 'CertificateAuthorityAccountResponse',
                                        204: 'CertificateAuthorityAccountResponse'
                                    }
                                )

                    class _connection(CloudApiEndpoint):
                        def __init__(self, *args, **kwargs):
                            super().__init__(*args, **kwargs)
                            self.validation = self._validation(api_obj=self._api_obj, url=f'{self._url}/validation')

                        class _validation(CloudApiEndpoint):
                            def post(
                                self,
                                CertificateAuthorityTestConnectionRequest: caoperations_service.CertificateAuthorityTestConnectionRequest
                            ):
                                data = {**CertificateAuthorityTestConnectionRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountResponse: caoperations_service.CertificateAuthorityAccountResponse

                                return generate_output(
                                    output_cls=Output,
                                    response=self._post(data=data),
                                    rc_mapping={
                                        200: 'CertificateAuthorityAccountResponse'
                                    }
                                )

                    class _credentials(CloudApiEndpoint):
                        def __init__(self, *args, **kwargs):
                            super().__init__(*args, **kwargs)
                            self.validation = self._validation(api_obj=self._api_obj, url=f'{self._url}/validation')

                        class _validation(CloudApiEndpoint):
                            def post(
                                self,
                                CertificateAuthorityCredentialsRequest: caoperations_service.CertificateAuthorityCredentialsRequest
                            ):
                                data = {**CertificateAuthorityCredentialsRequest.dict()}

                                class Output(CloudApiOutputModel):
                                    CertificateAuthorityAccountResponse: caoperations_service.CertificateAuthorityAccountResponse

                                return generate_output(
                                    output_cls=Output,
                                    response=self._post(data=data),
                                    rc_mapping={
                                        200: 'CertificateAuthorityAccountResponse'
                                    }
                                )

        class _certificateissuingtemplates(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            def get(self, certificateAuthorityAccountId: UUID):
                data = {
                    'certificateAuthorityAccountId': certificateAuthorityAccountId,
                }

                class Output(CloudApiOutputModel):
                    CertificateIssuingTemplateResponse: caoperations_service.CertificateIssuingTemplateResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params=data),
                    rc_mapping={
                        200: 'CertificateIssuingTemplateResponse'
                    }
                )

            def post(self, CertificateIssuingTemplateRequest: caoperations_service.CertificateIssuingTemplateRequest):
                data = {**CertificateIssuingTemplateRequest.dict()}

                class Output(CloudApiOutputModel):
                    CertificateIssuingTemplateResponse: caoperations_service.CertificateIssuingTemplateResponse

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        201: 'CertificateIssuingTemplateResponse'
                    }
                )

            class _ID(CloudApiEndpoint):
                def delete(self):
                    class Output(CloudApiOutputModel):
                        CertificateIssuingTemplateDeleteResponse: caoperations_service.CertificateIssuingTemplateDeleteResponse

                    return generate_output(
                        output_cls=Output,
                        response=self._delete(params={}),
                        rc_mapping={
                            204: 'CertificateIssuingTemplateDeleteResponse'
                        }
                    )

                def get(self):
                    class Output(CloudApiOutputModel):
                        CertificateIssuingTemplateInformation: caoperations_service.CertificateIssuingTemplateInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'CertificateIssuingTemplateInformation'
                        }
                    )

                def put(
                    self,
                    CertificateIssuingTemplateRequest: caoperations_service.CertificateIssuingTemplateRequest
                ):
                    data = {**CertificateIssuingTemplateRequest.dict()}

                    class Output(CloudApiOutputModel):
                        CertificateIssuingTemplateInformation: caoperations_service.CertificateIssuingTemplateInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._put(data=data),
                        rc_mapping={
                            200: 'CertificateIssuingTemplateInformation',
                            202: 'CertificateIssuingTemplateInformation'
                        }
                    )

        class _certificaterequests(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.approval = self._approval(api_obj=self._api_obj, url=f'{self._url}/approval')
                self.approvalrequests = self._approvalrequests(
                    api_obj=self._api_obj,
                    url=f'{self._url}/approvalrequests'
                )
                self.approvalrules = self._approvalrules(api_obj=self._api_obj, url=f'{self._url}/approvalrules')

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            class _ID(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.approval = self._approval(api_obj=self._api_obj, url=f'{self._url}/approval')

                class _approval(CloudApiEndpoint):
                    def __init__(self, *args, **kwargs):
                        super().__init__(*args, **kwargs)

                    def DECISION(self, decision: str):
                        return self._DECISION(api_obj=self._api_obj, url=f'{self._url}/{decision}')

                    class _DECISION(CloudApiEndpoint):
                        def post(self, ApprovalDecisionRequest: caoperations_service.ApprovalDecisionRequest):
                            data = {**ApprovalDecisionRequest.dict()}

                            class Output(CloudApiOutputModel):
                                CertificateRequestInformation: caoperations_service.CertificateRequestInformation

                            return generate_output(
                                output_cls=Output,
                                response=self._post(data=data),
                                rc_mapping={
                                    200: 'CertificateRequestInformation'
                                }
                            )

            class _approval(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.bulk = self._bulk(api_obj=self._api_obj, url=f'{self._url}/bulk')

                class _bulk(CloudApiEndpoint):
                    def __init__(self, *args, **kwargs):
                        super().__init__(*args, **kwargs)

                    def DECISION(self, decision: str):
                        return self._DECISION(api_obj=self._api_obj, url=f'{self._url}/{decision}')

                    def OPERATIONID(self, operationid: str):
                        return self._OPERATIONID(api_obj=self._api_obj, url=f'{self._url}/{operationid}')

                    class _DECISION(CloudApiEndpoint):
                        def post(self, BulkApprovalRequest: caoperations_service.BulkApprovalRequest):
                            data = {**BulkApprovalRequest.dict()}

                            class Output(CloudApiOutputModel):
                                BulkApprovalResponse: caoperations_service.BulkApprovalResponse

                            return generate_output(
                                output_cls=Output,
                                response=self._post(data=data),
                                rc_mapping={
                                    201: 'BulkApprovalResponse'
                                }
                            )

                    class _OPERATIONID(CloudApiEndpoint):
                        def get(self):
                            class Output(CloudApiOutputModel):
                                BulkApprovalStatusResponse: caoperations_service.BulkApprovalStatusResponse

                            return generate_output(
                                output_cls=Output,
                                response=self._get(params={}),
                                rc_mapping={
                                    200: 'BulkApprovalStatusResponse'
                                }
                            )

            class _approvalrequests(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def ENTITYID(self, entityid: str):
                    return self._ENTITYID(api_obj=self._api_obj, url=f'{self._url}/{entityid}')

                class _ENTITYID(CloudApiEndpoint):
                    def get(self):
                        class Output(CloudApiOutputModel):
                            ApprovalRequestInformation: caoperations_service.ApprovalRequestInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'ApprovalRequestInformation'
                            }
                        )

            class _approvalrules(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def ID(self, id: str):
                    return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                def get(self):
                    class Output(CloudApiOutputModel):
                        CertificateRequestApprovalRulesResponseOpenApi: caoperations_service.CertificateRequestApprovalRulesResponseOpenApi

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'CertificateRequestApprovalRulesResponseOpenApi'
                        }
                    )

                def post(
                    self,
                    CertificateRequestApprovalRulesRequest: caoperations_service.CertificateRequestApprovalRulesRequest
                ):
                    data = {**CertificateRequestApprovalRulesRequest.dict()}

                    class Output(CloudApiOutputModel):
                        CertificateRequestApprovalRuleOpenApi: caoperations_service.CertificateRequestApprovalRuleOpenApi

                    return generate_output(
                        output_cls=Output,
                        response=self._post(data=data),
                        rc_mapping={
                            201: 'CertificateRequestApprovalRuleOpenApi'
                        }
                    )

                class _ID(CloudApiEndpoint):
                    def delete(self):
                        class Output(CloudApiOutputModel):
                            CertificateRequestApprovalRuleDeleteResponseOpenApi: caoperations_service.CertificateRequestApprovalRuleDeleteResponseOpenApi

                        return generate_output(
                            output_cls=Output,
                            response=self._delete(params={}),
                            rc_mapping={
                                200: 'CertificateRequestApprovalRuleDeleteResponseOpenApi'
                            }
                        )

                    def get(self):
                        class Output(CloudApiOutputModel):
                            CertificateRequestApprovalRuleOpenApi: caoperations_service.CertificateRequestApprovalRuleOpenApi

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'CertificateRequestApprovalRuleOpenApi'
                            }
                        )

                    def put(
                        self,
                        CertificateRequestApprovalRulesUpdateRequest: caoperations_service.CertificateRequestApprovalRulesUpdateRequest
                    ):
                        data = {**CertificateRequestApprovalRulesUpdateRequest.dict()}

                        class Output(CloudApiOutputModel):
                            CertificateRequestApprovalRuleOpenApi: caoperations_service.CertificateRequestApprovalRuleOpenApi

                        return generate_output(
                            output_cls=Output,
                            response=self._put(data=data),
                            rc_mapping={
                                200: 'CertificateRequestApprovalRuleOpenApi'
                            }
                        )

        class _certificates(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.revocations = self._revocations(api_obj=self._api_obj, url=f'{self._url}/revocations')

            class _revocations(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.approvalrules = self._approvalrules(api_obj=self._api_obj, url=f'{self._url}/approvalrules')

                class _approvalrules(CloudApiEndpoint):
                    def __init__(self, *args, **kwargs):
                        super().__init__(*args, **kwargs)

                    def ID(self, id: str):
                        return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                    def get(self):
                        class Output(CloudApiOutputModel):
                            CertificateRevocationApprovalRulesResponseOpenApi: caoperations_service.CertificateRevocationApprovalRulesResponseOpenApi

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'CertificateRevocationApprovalRulesResponseOpenApi'
                            }
                        )

                    def post(
                        self,
                        CertificateRevocationApprovalRulesRequest: caoperations_service.CertificateRevocationApprovalRulesRequest
                    ):
                        data = {**CertificateRevocationApprovalRulesRequest.dict()}

                        class Output(CloudApiOutputModel):
                            CertificateRevocationApprovalRuleOpenApi: caoperations_service.CertificateRevocationApprovalRuleOpenApi

                        return generate_output(
                            output_cls=Output,
                            response=self._post(data=data),
                            rc_mapping={
                                201: 'CertificateRevocationApprovalRuleOpenApi'
                            }
                        )

                    class _ID(CloudApiEndpoint):
                        def delete(self):
                            class Output(CloudApiOutputModel):
                                CertificateRevocationApprovalRuleDeleteResponseOpenApi: caoperations_service.CertificateRevocationApprovalRuleDeleteResponseOpenApi

                            return generate_output(
                                output_cls=Output,
                                response=self._delete(params={}),
                                rc_mapping={
                                    200: 'CertificateRevocationApprovalRuleDeleteResponseOpenApi'
                                }
                            )

                        def get(self):
                            class Output(CloudApiOutputModel):
                                CertificateRevocationApprovalRuleOpenApi: caoperations_service.CertificateRevocationApprovalRuleOpenApi

                            return generate_output(
                                output_cls=Output,
                                response=self._get(params={}),
                                rc_mapping={
                                    200: 'CertificateRevocationApprovalRuleOpenApi'
                                }
                            )

                        def put(
                            self,
                            CertificateRevocationApprovalRulesUpdateRequest: caoperations_service.CertificateRevocationApprovalRulesUpdateRequest
                        ):
                            data = {**CertificateRevocationApprovalRulesUpdateRequest.dict()}

                            class Output(CloudApiOutputModel):
                                CertificateRevocationApprovalRuleOpenApi: caoperations_service.CertificateRevocationApprovalRuleOpenApi

                            return generate_output(
                                output_cls=Output,
                                response=self._put(data=data),
                                rc_mapping={
                                    200: 'CertificateRevocationApprovalRuleOpenApi'
                                }
                            )



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.organization import OrganizationAttributes

class AgentModuleHandlerContainerAttributes(OrganizationAttributes, metaclass=IterableMeta):
    __config_class__ = "Agent Module Handler Container"



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import (
    codesign,
    flow,
)

class _Flow(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Flow')
        self.Actions = self._Actions(api_obj=api_obj, url=f'{self._url}/Actions')
        self.Tickets = self._Tickets(api_obj=api_obj, url=f'{self._url}/Tickets')

    class _Actions(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.CodeSign = self._CodeSign(api_obj=self._api_obj, url=f'{self._url}/CodeSign')

        class _CodeSign(WebSdkEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.PreQualify = self._PreQualify(api_obj=self._api_obj, url=f'{self._url}/PreQualify')

            class _PreQualify(WebSdkEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.Create = self._Create(api_obj=self._api_obj, url=f'{self._url}/Create')

                class _Create(WebSdkEndpoint):
                    def post(
                        self, comment: str, data: str, dn: str, user: str, hours: int = None,
                        single_use: bool = None
                    ):
                        body = {
                            'Comment'  : comment,
                            'Data'     : data,
                            'Dn'       : dn,
                            'Hours'    : hours,
                            'SingleUse': single_use,
                            'User'     : user
                        }

                        class Output(WebSdkOutputModel):
                            result: codesign.ResultCode = ApiField(
                                alias='Result', converter=lambda x: codesign.ResultCode(code=x)
                            )
                            success: bool = ApiField(alias='Success')

                        return generate_output(output_cls=Output, response=self._post(data=body))

    class _Tickets(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.Approve = self._Approve(api_obj=self._api_obj, url=f'{self._url}/Approve')
            self.Count = self._Count(api_obj=self._api_obj, url=f'{self._url}/Count')
            self.CountApproved = self._CountApproved(api_obj=self._api_obj, url=f'{self._url}/CountApproved')
            self.Enumerate = self._Enumerate(api_obj=self._api_obj, url=f'{self._url}/Enumerate')
            self.EnumerateApproved = self._EnumerateApproved(
                api_obj=self._api_obj,
                url=f'{self._url}/EnumerateApproved'
            )
            self.Load = self._Load(api_obj=self._api_obj, url=f'{self._url}/Load')
            self.Reject = self._Reject(api_obj=self._api_obj, url=f'{self._url}/Reject')
            self.Update = self._Update(api_obj=self._api_obj, url=f'{self._url}/Update')

        class _Approve(WebSdkEndpoint):
            def post(
                self, ticket_id: int = None, ticket_ids: list[int] = None, expires: str = None,
                comment: str = None, not_before: str = None, use_count: int = None
            ):
                body = {
                    'Expires'  : expires,
                    'Comment'  : comment,
                    'notBefore': not_before,
                    'TicketId' : ticket_id,
                    'TicketIds': ticket_ids,
                    'useCount' : use_count
                }

                class Output(WebSdkOutputModel):
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))
                    message: str = ApiField(alias='Message')

                return generate_output(output_cls=Output, response=self._post(data=body))

        class _Count(WebSdkEndpoint):
            def post(self):
                class Output(WebSdkOutputModel):
                    count: int = ApiField(alias='Count')
                    message: str = ApiField(alias='Message')
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))

                return generate_output(output_cls=Output, response=self._post(data={}))

        class _CountApproved(WebSdkEndpoint):
            def post(self):
                class Output(WebSdkOutputModel):
                    count: int = ApiField(alias='Count')
                    message: str = ApiField(alias='Message')
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))

                return generate_output(output_cls=Output, response=self._post(data={}))

        class _Enumerate(WebSdkEndpoint):
            def post(self, product_code: int = None, ticket_page_size: int = None, ticket_page_number: int = None):
                body = {
                    'ProductCode'     : product_code,
                    'TicketPageSize'  : ticket_page_size,
                    'TicketPageNumber': ticket_page_number
                }

                class Output(WebSdkOutputModel):
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))
                    message: str = ApiField(alias='Message')
                    tickets: list[flow.Ticket] = ApiField(alias='Tickets', default_factory=list)

                return generate_output(output_cls=Output, response=self._post(data=body))

        class _EnumerateApproved(WebSdkEndpoint):
            def post(self, product_code: int = None, ticket_page_size: int = None, ticket_page_number: int = None):
                body = {
                    'ProductCode'     : product_code,
                    'TicketPageSize'  : ticket_page_size,
                    'TicketPageNumber': ticket_page_number
                }

                class Output(WebSdkOutputModel):
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))
                    message: str = ApiField(alias='Message')
                    tickets: list[flow.Ticket] = ApiField(alias='Tickets', default_factory=list)

                return generate_output(output_cls=Output, response=self._post(data=body))

        class _Load(WebSdkEndpoint):
            def post(self, ticket_id: int = None, ticket_ids: list[int] = None):
                body = {
                    'TicketId' : ticket_id,
                    'TicketIds': ticket_ids
                }

                class Output(WebSdkOutputModel):
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))
                    message: str = ApiField(alias='Message')
                    tickets: list[flow.Ticket] = ApiField(alias='Tickets', default_factory=list)

                return generate_output(output_cls=Output, response=self._post(data=body))

        class _Reject(WebSdkEndpoint):
            def post(
                self, comment: str, rejection_level: int, ticket_id: int = None,
                ticket_ids: list[int] = None
            ):
                body = {
                    'Comment'       : comment,
                    'RejectionLevel': rejection_level,
                    'TicketId'      : ticket_id,
                    'TicketIds'     : ticket_ids
                }

                class Output(WebSdkOutputModel):
                    invalid_ticket_ids: list[int] = ApiField(default_factory=list, alias='InvalidTicketIds')
                    message: str = ApiField(alias='Message')
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))

                return generate_output(output_cls=Output, response=self._post(data=body))

        class _Update(WebSdkEndpoint):
            def post(
                self, ticket_id: int = None, ticket_ids: list[int] = None, expires: str = None,
                comment: str = None, not_before: str = None, use_count: int = None
            ):
                body = {
                    'Expires'  : expires,
                    'Comment'  : comment,
                    'notBefore': not_before,
                    'TicketId' : ticket_id,
                    'TicketIds': ticket_ids,
                    'useCount' : use_count
                }

                class Output(WebSdkOutputModel):
                    result: flow.Result = ApiField(alias='Result', converter=lambda x: flow.Result(code=x))
                    message: str = ApiField(alias='Message')

                return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_trust_store_base import CertificateTrustStoreBaseAttributes

class DataPowerTrustStoreAttributes(CertificateTrustStoreBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "DataPower Trust Store"
    application_domain = Attribute('Application Domain', min_version='21.4')
    crl_distribution_points_handling = Attribute('CRL Distribution Points Handling', min_version='21.4')
    certificate_validation_mode = Attribute('Certificate Validation Mode', min_version='21.4')
    check_dates = Attribute('Check Dates', min_version='21.4')
    crypto_validation_credential_name = Attribute('Crypto Validation Credential Name', min_version='21.4')
    folder = Attribute('Folder', min_version='21.4')
    maximum_length = Attribute('Maximum Length', min_version='21.4')
    require_crl = Attribute('Require CRL', min_version='21.4')
    use_crl = Attribute('Use CRL', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.validation_base import ValidationBaseAttributes

class X509CertificateValidationAttributes(ValidationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "X509 Certificate Validation"
    detect_all_ssl_tls_protocols = Attribute('Detect All SSL TLS Protocols', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    use_common_name = Attribute('Use Common Name', min_version='21.4')
    use_dns_subjectaltname = Attribute('Use DNS SubjectAltName', min_version='21.4')
    validate_chain_returned_by_host = Attribute('Validate Chain Returned By Host', min_version='21.4')



from __future__ import annotations

from datetime import datetime
from typing import (
    Literal,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)

CertificateFormat = Literal['Base64', 'Base64 (PKCS #8)', 'DER', 'JKS', 'PKCS #7', 'PKCS #12']

class Link(ObjectModel):
    details: str = ApiField(alias='Details')
    next: str = ApiField(alias='Next')
    previous: str = ApiField(alias='Previous')

class CertificateDetails(ObjectModel):
    aia_ca_issuer_url: list[str] = ApiField(alias='AIACAIssuerURL', default_factory=list)
    aia_key_identifier: str = ApiField(alias='AIAKeyIdentifier')
    c: str = ApiField(alias='C')
    cdp_uri: str = ApiField(alias='CDPURI')
    cn: str = ApiField(alias='CN')
    elliptic_curve: str = ApiField(alias='EllipticCurve')
    enhanced_key_usage: str = ApiField(alias='EnhancedKeyUsage')
    issuer: str = ApiField(alias='Issuer')
    key_algorithm: str = ApiField(alias='KeyAlgorithm')
    key_size: int = ApiField(alias='KeySize')
    key_usage: str = ApiField(alias='KeyUsage')
    l: str = ApiField(alias='L')
    o: str = ApiField(alias='O')
    ou: list[str] = ApiField(alias='OU', default_factory=list)
    public_key_hash: str = ApiField(alias='PublicKeyHash')
    revocation_date: datetime = ApiField(alias='RevocationDate')
    revocation_status: str = ApiField(alias='RevocationStatus')
    s: str = ApiField(alias='S')
    ski_key_identifier: str = ApiField(alias='SKIKeyIdentifier')
    serial: str = ApiField(alias='Serial')
    signature_algorithm: str = ApiField(alias='SignatureAlgorithm')
    signature_algorithm_oid: str = ApiField(alias='SignatureAlgorithmOID')
    store_added: datetime = ApiField(alias='StoreAdded')
    subject: str = ApiField(alias='Subject')
    subject_alt_name_dns: list[str] = ApiField(alias='SubjectAltNameDNS', default_factory=list)
    subject_alt_name_email: list[str] = ApiField(alias='SubjectAltNameEmail', default_factory=list)
    subject_alt_name_ip_address: list[str] = ApiField(alias='SubjectAltNameIPAddress', default_factory=list)
    subject_alt_name_other_name_upn: list[str] = ApiField(alias='SubjectAltNameOtherNameUPN', default_factory=list)
    subject_alt_name_uri: list[str] = ApiField(alias='SubjectAltNameUri', default_factory=list)
    template_major_version: str = ApiField(alias='TemplateMajorVersion')
    template_minor_version: str = ApiField(alias='TemplateMajorVersion')
    template_name: str = ApiField(alias='TemplateName')
    template_oid: str = ApiField(alias='TemplateOID')
    thumbprint: str = ApiField(alias='Thumbprint')
    valid_from: datetime = ApiField(alias='ValidFrom')
    valid_to: datetime = ApiField(alias='ValidTo')

class PreviousVersions(ObjectModel):
    certificate_details: CertificateDetails = ApiField(alias='CertificateDetails')
    vault_id: int = ApiField(alias='VaultId')

class ProcessingDetails(ObjectModel):
    in_error: bool = ApiField(alias='InError')
    stage: int = ApiField(alias='Stage')
    status: str = ApiField(alias='Status')

class RenewalDetails(ObjectModel):
    city: str = ApiField(alias='City')
    country: str = ApiField(alias='Country')
    organization: str = ApiField(alias='Organization')
    organizational_unit: list[str] = ApiField(alias='OrganizationalUnit', default_factory=list)
    state: str = ApiField(alias='State')
    subject: str = ApiField(alias='Subject')
    subject_alt_name_dns: list[str] = ApiField(alias='SubjectAltNameDns', default_factory=list)
    subject_alt_name_email: list[str] = ApiField(alias='SubjectAltNameEmail', default_factory=list)
    subject_alt_name_ip_address: list[str] = ApiField(alias='SubjectAltNameIpAddress', default_factory=list)
    subject_alt_name_other_name_upn: list[str] = ApiField(alias='SubjectAltNameOtherNameUpn', default_factory=list)
    subject_alt_name_uri: list[str] = ApiField(alias='SubjectAltNameUri', default_factory=list)
    valid_from: datetime = ApiField(alias='ValidFrom')
    valid_to: datetime = ApiField(alias='ValidTo')

class ValidationDetails(ObjectModel):
    last_validation_state_update: str = ApiField(alias='LastValidationStateUpdate')
    validation_state: str = ApiField(alias='ValidationState')

class File(ObjectModel):
    installation: str = ApiField(alias='Installation')
    performed_on: datetime = ApiField(alias='PerformedOn')
    result: list[str] = ApiField(alias='Result', default_factory=list)

class BitMaskValues(ObjectModel):
    bitmask: int = ApiField(alias='Bitmask')
    values: list[str] = ApiField(alias='Values', default_factory=list)

class SANS(ObjectModel):
    dns: list[str] = ApiField(alias='Dns', default_factory=list)
    ip: list[str] = ApiField(alias='Ip', default_factory=list)

class Compliant(ObjectModel):
    compliant: bool = ApiField(alias='Compliant')

class CompliantSingleValue(Compliant):
    value: str = ApiField(alias='Value')

class CompliantMultiValue(Compliant):
    values: list[str] = ApiField(alias='Values', default_factory=list)

class Locked(ObjectModel):
    locked: bool = ApiField(alias='Locked')

class LockedSingleValue(Locked):
    value: str = ApiField(alias='Value')

class LockedMultiValue(Locked):
    values: list = ApiField(alias='Values')

class LockedKeyPair(ObjectModel):
    key_algorithm: LockedSingleValue = ApiField(alias='KeyAlgorithm')
    key_size: LockedSingleValue = ApiField(alias='KeySize')

class LockedSubject(ObjectModel):
    city: LockedSingleValue = ApiField(alias='City')
    country: LockedSingleValue = ApiField(alias='Country')
    organization: LockedSingleValue = ApiField(alias='Organization')
    organizational_units: LockedMultiValue = ApiField(alias='OrganizationalUnits')
    state: LockedSingleValue = ApiField(alias='State')

class CSRDetails(ObjectModel):
    city: CompliantSingleValue = ApiField(alias='City')
    common_name: CompliantSingleValue = ApiField(alias='CommonName')
    country: CompliantSingleValue = ApiField(alias='Country')
    key_algorithm: CompliantSingleValue = ApiField(alias='KeyAlgorithm')
    key_size: CompliantSingleValue = ApiField(alias='KeySize')
    organization: CompliantSingleValue = ApiField(alias='Organization')
    organizational_unit: CompliantMultiValue = ApiField(alias='OrganizationalUnit')
    private_key_reused: CompliantSingleValue = ApiField(alias='PrivateKeyReused')
    state: CompliantSingleValue = ApiField(alias='State')
    subj_alt_name_dns: CompliantMultiValue = ApiField(alias='SubjAltNameDns')
    subj_alt_name_email: CompliantMultiValue = ApiField(alias='SubjAltNameEmail')
    subj_alt_name_ip: CompliantMultiValue = ApiField(alias='SubjAltNameIp')
    subj_alt_name_upn: CompliantMultiValue = ApiField(alias='SubjAltNameUpn')
    subj_alt_name_uri: CompliantMultiValue = ApiField(alias='SubjAltNameUri')

class NameValue(ObjectModel):
    name: str = ApiField(alias='Name')
    value: str = ApiField(alias='Value')

class NameTypeValue(ObjectModel):
    type: str = ApiField(alias='Type')
    name: str = ApiField(alias='Name')
    value: str = ApiField(alias='Value')

class SslTlsResult(ObjectModel):
    chain: BitMaskValues = ApiField(alias='Chain')
    end_entity: BitMaskValues = ApiField(alias='EndEntity')
    id: int = ApiField(alias='Id')
    protocols: BitMaskValues = ApiField(alias='Protocols')

class SslTls(ObjectModel):
    host: str = ApiField(alias='Host')
    ip_address: str = ApiField(alias='IpAddress')
    port: int = ApiField(alias='Port')
    result: SslTlsResult = ApiField(alias='Result')
    sources: list[str] = ApiField(alias='Sources', default_factory=list)

class Policy(ObjectModel):
    certificate_authority: LockedSingleValue = ApiField(alias='CertificateAuthority')
    csr_generation: LockedSingleValue = ApiField(alias='CsrGeneration')
    management_type: LockedSingleValue = ApiField(alias='ManagementType')
    key_generation: LockedSingleValue = ApiField(alias='KeyGeneration')
    key_pair: LockedKeyPair = ApiField(alias='KeyPair')
    private_key_reuse_allowed: bool = ApiField(alias='PrivateKeyReuseAllowed')
    subj_alt_name_dns_allowed: bool = ApiField(alias='SubjAltNameDnsAllowed')
    subj_alt_name_email_allowed: bool = ApiField(alias='SubjAltNameEmailAllowed')
    subj_alt_name_ip_allowed: bool = ApiField(alias='SubjAltNameIpAllowed')
    subj_alt_name_upn_allowed: bool = ApiField(alias='SubjAltNameUpnAllowed')
    subj_alt_name_uri_allowed: bool = ApiField(alias='SubjAltNameUriAllowed')
    subject: LockedSubject = ApiField(alias='Subject')
    unique_subject_enforced: bool = ApiField(alias='UniqueSubjectEnforced')
    whitelisted_domains: list[str] = ApiField(alias='WhitelistedDomains', default_factory=list)
    wildcards_allowed: bool = ApiField(alias='WildcardsAllowed')

class CSR(ObjectModel):
    details: CSRDetails = ApiField(alias='Details')
    enrollable: bool = ApiField(alias='Enrollable')

class X509(ObjectModel):
    cn: str = ApiField(alias='Cn')
    issuer: str = ApiField(alias='Issuer')
    key_algorithm: str = ApiField(alias='KeyAlgorithm')
    key_size: int = ApiField(alias='KeySize')
    sans: SANS = ApiField(alias='Sans')
    serial: str = ApiField(alias='Serial')
    subject: str = ApiField(alias='Subject')
    thumbprint: str = ApiField(alias='Thumbprint')
    valid_from: datetime = ApiField(alias='ValidFrom')
    valid_to: datetime = ApiField(alias='ValidTo')

class Certificate(ObjectModel):
    created_on: datetime = ApiField(alias='CreatedOn')
    dn: str = ApiField(alias='DN')
    guid: str = ApiField(alias='Guid')
    name: str = ApiField(alias='Name')
    parent_dn: str = ApiField(alias='ParentDn')
    schema_class: str = ApiField(alias='SchemaClass')
    x509: X509 = ApiField(alias='X509')
    links: list[Link] = ApiField(alias='_links', default_factory=list)

class CertificateFilter(ObjectModel):
    certificate_type: str = ApiField(alias='CertificateType')
    chain_validation_failure: str = ApiField(alias='ChainValidationFailure')
    cn: str = ApiField(alias='CN')
    country: str = ApiField(alias='C')
    created_on: datetime = ApiField(alias='CreatedOn')
    created_on_greater: datetime = ApiField(alias='CreatedOnGreater')
    created_on_less: datetime = ApiField(alias='CreatedOnLess')
    disabled: bool = ApiField(alias='Disabled')
    in_error: bool = ApiField(alias='InError')
    is_self_signed: bool = ApiField(alias='IsSelfSigned')
    is_wildcard: bool = ApiField(alias='IsWildcard')
    issuer: str = ApiField(alias='Issuer')
    key_algorithm: str = ApiField(alias='KeyAlgorithm')
    key_size: str = ApiField(alias='KeySize')
    key_size_greater: str = ApiField(alias='KeySizeGreater')
    key_size_less: str = ApiField(alias='KeySizeLess')
    l: str = ApiField(alias='L')
    management_type: str = ApiField(alias='ManagementType')
    name: str = ApiField(alias='Name')
    network_validation_disabled: str = ApiField(alias='NetworkValidationDisabled')
    o: str = ApiField(alias='O')
    ou: str = ApiField(alias='OU')
    parent_dn: str = ApiField(alias='ParentDn')
    parent_dn_recursive: str = ApiField(alias='ParentDnRecursive')
    pending_workflow: bool = ApiField(alias='PendingWorkflow')
    s: str = ApiField(alias='S')
    san_dns: str = ApiField(alias='SAN-DNS')
    san_email: str = ApiField(alias='SAN-Email')
    san_ip: str = ApiField(alias='SAN-IP')
    san_upn: str = ApiField(alias='SAN-UPN')
    san_uri: str = ApiField(alias='SAN-URI')
    serial: str = ApiField(alias='Serial')
    signature_algorithm: str = ApiField(alias='SignatureAlgorithm')
    ssl_tls_protocol: str = ApiField(alias='SslTlsProtocol')
    stage: str = ApiField(alias='Stage')
    stage_greater: str = ApiField(alias='StageGreater')
    stage_less: str = ApiField(alias='StageLess')
    thumbprint: str = ApiField(alias='Thumbprint')
    tls_validation_failure: str = ApiField(alias='TlsValidationFailure')
    valid_from: datetime = ApiField(alias='ValidFrom')
    valid_from_greater: datetime = ApiField(alias='ValidFromGreater')
    valid_from_less: datetime = ApiField(alias='ValidFromLess')
    valid_to: datetime = ApiField(alias='ValidTo')
    valid_to_greater: datetime = ApiField(alias='ValidToGreater')
    valid_to_less: datetime = ApiField(alias='ValidToLess')
    validation_disabled: bool = ApiField(alias='ValidationDisabled')
    validation_state: str = ApiField(alias='ValidationState')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.top import TopAttributes

class RemoteAccessScopeAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Remote Access Scope"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class X509RootCertificateAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "X509 Root Certificate"
    aia_ca_issuer_location_description = Attribute('AIA CA Issuer Location Description')
    aia_ca_issuer_location_disabled = Attribute('AIA CA Issuer Location Disabled')
    aia_ca_issuer_location_mode = Attribute('AIA CA Issuer Location Mode')
    aia_ca_issuer_location_rank = Attribute('AIA CA Issuer Location Rank')
    aia_ca_issuer_location_status = Attribute('AIA CA Issuer Location Status')
    aia_ca_issuer_publishing_location = Attribute('AIA CA Issuer Publishing Location')
    aia_ocsp_location_description = Attribute('AIA OCSP Location Description')
    aia_ocsp_location_disabled = Attribute('AIA OCSP Location Disabled')
    aia_ocsp_location_mode = Attribute('AIA OCSP Location Mode')
    aia_ocsp_location_rank = Attribute('AIA OCSP Location Rank')
    aia_ocsp_location_status = Attribute('AIA OCSP Location Status')
    aia_ocsp_publishing_location = Attribute('AIA OCSP Publishing Location')
    cdp_aia_actions = Attribute('CDP AIA Actions')
    cdp_aia_in_error = Attribute('CDP AIA In Error')
    cdp_aia_status = Attribute('CDP AIA Status')
    cdp_aia_verification_now = Attribute('CDP AIA Verification Now')
    cdp_crl_metadata = Attribute('CDP CRL Metadata')
    cdp_crl_next_publish_percentage = Attribute('CDP CRL Next Publish Percentage')
    cdp_location_description = Attribute('CDP Location Description')
    cdp_location_disabled = Attribute('CDP Location Disabled')
    cdp_location_mode = Attribute('CDP Location Mode')
    cdp_location_rank = Attribute('CDP Location Rank')
    cdp_location_status = Attribute('CDP Location Status')
    cdp_publishing_location = Attribute('CDP Publishing Location')
    crl_delta_number = Attribute('CRL Delta Number')
    crl_delta_publishing_location = Attribute('CRL Delta Publishing Location')
    crl_issuer_dn = Attribute('CRL Issuer DN')
    crl_issuer_identifier = Attribute('CRL Issuer Identifier')
    crl_next_publish_date = Attribute('CRL Next Publish Date')
    crl_next_update = Attribute('CRL Next Update')
    crl_number = Attribute('CRL Number')
    crl_this_update = Attribute('CRL This Update')
    certificate_vault_id = Attribute('Certificate Vault Id')
    discovered_on = Attribute('Discovered On')
    escalation_notice_interval = Attribute('Escalation Notice Interval')
    escalation_notice_start = Attribute('Escalation Notice Start')
    expiration_notice_interval = Attribute('Expiration Notice Interval')
    expiration_notice_start = Attribute('Expiration Notice Start')
    monitored_uri_verification_engine_info = Attribute('Monitored URI Verification Engine Info')
    monitored_uri_verification_last_check = Attribute('Monitored URI Verification Last Check')
    monitored_uri_verification_last_notification = Attribute('Monitored URI Verification Last Notification')
    monitored_uri_verification_run_identifier = Attribute('Monitored URI Verification Run Identifier')
    monitored_uri_verification_uri_identifier = Attribute('Monitored URI Verification URI Identifier')
    required_attribute_for_signing_certificate = Attribute('Required Attribute For Signing Certificate')
    required_attribute_for_time_stamping_certificate = Attribute('Required Attribute For Time Stamping Certificate')
    trust_for_signing = Attribute('Trust For Signing')
    trust_for_time_stamping = Attribute('Trust For Time Stamping')
    trusted_status = Attribute('Trusted Status')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.cloud_instance_monitor_driver_base import CloudInstanceMonitorDriverBaseAttributes

class AWSEC2InstanceMonitorAttributes(CloudInstanceMonitorDriverBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "AWS EC2 Instance Monitor"
    access_key_credential_dn = Attribute('Access Key Credential DN', min_version='21.4')
    aws_credential_dn = Attribute('Aws Credential DN', min_version='21.4')
    secret_key_credential_dn = Attribute('Secret Key Credential DN', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class TealeafPCAAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Tealeaf PCA"
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    install_path = Attribute('Install Path', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes
from pyvenafi.tpp.attributes.proxy import ProxyAttributes

class zOSCAAttributes(HTTPCABaseAttributes, ProxyAttributes, metaclass=IterableMeta):
    __config_class__ = "zOS CA"
    client_port = Attribute('Client Port')
    client_secure = Attribute('Client Secure')
    domain = Attribute('Domain')
    secure = Attribute('Secure')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class AzureKeyVaultAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Azure Key Vault"
    app_service_subscription_id = Attribute('App Service Subscription ID', min_version='22.1')
    binding_hostnames = Attribute('Binding Hostnames', min_version='21.4')
    binding_ssl_type = Attribute('Binding SSL Type', min_version='21.4')
    certificate_credential = Attribute('Certificate Credential', min_version='21.4')
    certificate_name = Attribute('Certificate Name', min_version='21.4')
    client_id = Attribute('Client ID', min_version='21.4')
    create_binding = Attribute('Create Binding', min_version='21.4')
    create_san_dns_bindings = Attribute('Create SAN DNS Bindings', min_version='21.4')
    environment = Attribute('Environment', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    non_exportable = Attribute('Non-Exportable', min_version='21.4')
    timeout = Attribute('Timeout', min_version='21.4')
    update_web_app = Attribute('Update Web App', min_version='21.4')
    vault_name = Attribute('Vault Name', min_version='21.4')
    web_app_name = Attribute('Web App Name', min_version='21.4')
